package bmsi.xopen;
import sun.io.ByteToCharConverter;
import sun.io.CharToByteConverter;
import sun.io.MalformedInputException;
import java.io.UnsupportedEncodingException;
import java.util.Hashtable;

/** This class is used to load and store values in Cisam/Xopen file format.
    @author Stuart D. Gathman
 */
public class FType {
  private final ByteToCharConverter tochar;
  private final CharToByteConverter tobyte;
  public final byte blank;
  private final String enc;
  private static final char[] cblank = { ' ' };
  private static final Hashtable tbl = new Hashtable();

  public static FType getConverter(String enc)
    throws UnsupportedEncodingException
  {
    FType f = (FType)tbl.get(enc);
    if (f == null) {
      f = new FType(enc);
      tbl.put(enc,f);
    }
    return f;
  }

  /** We cache the Converter because JDK1.1.x is horribly inefficient
      otherwise (since they fail to do so).  Although accessing the sun.io.*
      classes directly is not portable, it is worth the measured 4 four fold
      performance improvement to do so anyway.  I sure hope Javasoft
      makes the Converter API available again soon!  (They removed it
      after JDK1.1.1 promising an improved version "soon".)
   */
  private FType(String enc) throws UnsupportedEncodingException {
    ByteToCharConverter tochar = ByteToCharConverter.getConverter(enc);
    CharToByteConverter tobyte = CharToByteConverter.getConverter(enc);
    try {
      byte[] b = tobyte.convertAll(cblank);
      if (b.length != 1)
	throw new UnsupportedEncodingException(
	  "Multi-byte blank not supported");
      blank = b[0];
      this.tochar = tochar;
      this.tobyte = tobyte;
      this.enc = enc;
    }
    catch (MalformedInputException e) {
      throw new UnsupportedEncodingException("Can't convert blank");
    }
  }

  public String getEncoding() { return enc; }

  public final byte[] getBytes(String s) {
    try {
      return tobyte.convertAll(s.toCharArray());
    }
    catch (MalformedInputException e) {
      return new byte[0];
    }
  }

  public void ststr(String s,byte[] buf,int pos,int len) {
    ststr(s,buf,pos,len,blank);
  }

  public void ststr(String s,byte[] buf,int pos,int len,byte pad) {
    byte[] b = getBytes(s);
    while (len > b.length)
      buf[pos + --len] = pad;
    System.arraycopy(b,0,buf,pos,len);
  }

  public char[] convertAll(byte[] buf) {
    try {
      return tochar.convertAll(buf);
    }
    catch (MalformedInputException e) {
      return null;
    }
  }

  public byte[] convertAll(char[] c) {
    try {
      return tobyte.convertAll(c);
    }
    catch (MalformedInputException e) {
      return null;
    }
  }

  public String getString(byte[] buf) {
    try {
      return new String(tochar.convertAll(buf));
    }
    catch (MalformedInputException e) {
      return "";
    }
  }

  public String ldstr(byte[] buf,int pos,int len) {
    byte[] b = new byte[len];
    System.arraycopy(buf,pos,b,0,len);
    try {
      char[] c = tochar.convertAll(b);
      len = c.length;
      while (len > 0 && c[len-1] <= ' ') --len;
      return new String(c,0,len);
    }
    catch (MalformedInputException e) {
      return "";
    }
  }

  public static void stshort(short i,byte[] buf,int pos) {
    buf[pos++] = (byte)(i >> 8);
    buf[pos] = (byte)i; 
  }
  public static void stint(int i,byte[] buf,int pos) {
    buf[pos++] = (byte)(i >> 24);
    buf[pos++] = (byte)(i >> 16);
    buf[pos++] = (byte)(i >> 8);
    buf[pos] = (byte)i; 
  }

  public static void stnum(long v,byte[] buf,int pos,int len) {
    pos += len;
    while (--len >= 0) {
      buf[--pos] = (byte)v;
      v >>= 8;
    }
  }

  public static short ldshort(byte[] buf,int pos) {
    return (short)((buf[pos] << 8) | (buf[pos+1] & 255));
  }

  public static int ldushort(byte[] buf,int pos) {
    return ((buf[pos] & 255) << 8) | (buf[pos+1] & 255);
  }

  public static int ldint(byte[] buf,int pos) {
    return (ldshort(buf,pos) << 16) | ldushort(buf,pos+2);
  }

  public static long ldnum(byte[] buf,int pos,int len) {
    if (len <= 0) return 0;
    long v = buf[pos];
    while (--len > 0)
      v = (v << 8) | (buf[++pos] & 255);
    return v;
  }
}
