/* $Log$
 */
package bmsi.util;

public class IntSet {
  private int[] a = new int[8];
  private int size = 0;
  private int sorted = 0;

  private void sort() {
    if (sorted < size) {
      for (int gap = size/2; gap > 0; gap /= 2) {
	for (int i = gap; i < size; ++i) {
	  for (int j = i - gap; j >= 0; j -= gap) {
	    int p = j + gap;
	    if (a[p] >= a[j]) break;
	    int temp = a[j];
	    a[j] = a[p];
	    a[p] = temp;
	  }
	}
      }
    }
    sorted = size;
  }

  /** Search for n in the set.
    @return the index of n in the set.
   */
  private int search(int n) {
    // search the sorted portion
    int hi = sorted;
    int lo = 0;
    while (hi > lo) {
      int i = (hi + lo)/2;
      if (n == a[i]) return i;
      if (n < a[i])
        hi = i;
      else
        lo = i + 1;
    }
    // search the unsorted portion
    for (int i = sorted; i < size; ++i)
      if (a[i] == n) return i;
    return -1;
  }

  /** Return true if this set contains the integer i. */
  public boolean contains(int i) {
    return search(i) >= 0;
  }

  /** Return the number of integers in the set. */
  public final int size() { return size; }

  /** Add the integer i to this set. */
  public void add(int i) {
    if (search(i) < 0) {
      if (size >= a.length) {
	int[] na = new int[a.length * 2];
	System.arraycopy(a,0,na,0,size);
	a = na;
      }
      if (sorted <= 0)
	sorted = 1;
      else if (i > a[sorted-1]) {
	if (size > sorted)
	  a[size++] = a[sorted];
	else
	  ++size;
	a[sorted++] = i;
	return;
      }
      a[size++] = i;
    }
  }

  /** Remove the integer n from this set. */
  public boolean remove(int n) {
    int i = search(n);
    if (i < 0) return false;
    if (i < sorted)
      --sorted;
    if (i < --size)
      System.arraycopy(a,i+1,a,i,size - i);
    return true;
  }

  /** Remove all integers, making this an empty set. */
  public void removeAll() { size = 0; sorted = 0; }

  /** Return a sorted array of the integers in this set. */
  public int[] elements() {
    sort();
    int[] na = new int[size];
    System.arraycopy(a,0,na,0,size);
    return na;
  }

  /** Return the first integer added to this set. 
     This will be the integer with the lowest value.
   */
  public final int firstElement() {
    sort();
    return a[0];
  }

  /** Return true if this set is empty. */
  public final boolean isEmpty() { return size == 0; }

  public int hashCode() {
    int sum = 0;
    for (int i = 0; i < size; ++i)
      sum += a[i];
    return sum;
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof IntSet)) return false;
    IntSet s = (IntSet)obj;
    if (s.size != size) return false;
    sort();
    s.sort();
    for (int i = 0; i < size; ++i)
      if (s.a[i] != a[i]) return false;
    return true;
  }

  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("IntSet{");
    if (size > 0) {
      sort();
      buf.append(a[0]);
      for (int i = 1; i < size; ++i) {
        buf.append(',');
	buf.append(a[i]);
      }
    }
    buf.append('}');
    return buf.toString();
  }

}
