package bmsi.tui;
import java.awt.*;
import java.awt.peer.*;

abstract class TUIContainer extends TUIComponent {
  protected boolean userpaint =  true;
  protected TUIContainer(Container c,TUIKit toolkit) { super(c,toolkit); }
  /* needed in JDK1.0
  protected void initialize() {
    super.initialize();
    Container c = (Container)target;
    Component[] comp = c.getComponents();
    for (int i = 0; i < comp.length; ++i) {
      TUIComponent cp = (TUIComponent)comp[i].getPeer();
      if (cp != null)
	add(cp);
    }
  }
  */
  private boolean firstpaint = true;
  public void setVisible(boolean vis) {
    super.setVisible(vis);
    if (userpaint && vis && firstpaint) {
      firstpaint = false;
      //target.repaint();
      Graphics g = getGraphics();
      try {
	target.update(g);
	((Container)target).paintComponents(g);
      }
      finally {
	g.dispose();
      }
      //System.err.println("firstpaint " + winID);
    }
  }
  public Insets getInsets() { return new Insets(0,0,0,0); }
  public void add(TUIComponent c) {
    toolkit.writeCmd(this,ADDCOMPONENT,c.winID);
  }

  /** The purpose of beginValidate is not documented, but I *think*
   we are supposed to stop sending reshape commands for contained
   components until the Container finishes fiddling around and calls
   endValidate.  We'll ignore it for now since this is just an optimization.
   */
  public void beginValidate() { }
  public void endValidate() { }

  /** Add rectangle to list of areas to be repainted in ms time.
   * Most TUIComponents are painted entirely by the remote program.
   * A Panel is painted by the application only through a derived class.
   */
  public void repaint(long ms,int x,int y,int w,int h) {
    if (userpaint) {
      Graphics g = getGraphics();
      g.clipRect(x,y,w,h);
      target.update(g);
      g.dispose();
    }
  }

  /* jdk 1.0 compatibility */
  public final Insets insets() { return getInsets(); }
}
